@@ -80,26 +80,6 @@ static char *strlower(char *s)
 
 }
 
-/*----------------------------------------------------------------------------*/
-/*----------------------------------------------------------------------------*/
-/*
-
-static char *strupper(char *s)
-{
-
-  char *t = s;
-
-  while (*t) {
-    *t = toupper(*t);
-    t++;
-  }
-  
-  return s;
-
-}
-
-*/
-
 /*----------------------------------------------------------------------------*/
 /*----------------------------------------------------------------------------*/
 char *dfs_generate_filename(const char *category)
@@ -112,25 +92,6 @@ char *dfs_generate_filename(const char *category)
     return filename;
 }
 
-/*----------------------------------------------------------------------------*/
-/*----------------------------------------------------------------------------*/
-char *dfs_generate_filename_tfits(const char *category)
-{
-    /*
-    char *filename = cpl_calloc(strlen(category) + 7, sizeof(char));
-    */
-    char *filename = cpl_calloc(strlen(category) + 6, sizeof(char));
-
-    strlower(strcpy(filename, category));
-
-    /*
-    strcat(filename, ".tfits");
-    */
-    strcat(filename, ".fits");
-
-    return filename;
-}
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Dump a single CPL error
@@ -315,7 +276,16 @@ fors_dfs_set_groups(cpl_frameset * set)
                 strcmp(tag, "SCREEN_FLAT_PMOS") == 0 ||
                 strcmp(tag, "STANDARD_PMOS"   ) == 0 ||
                 strcmp(tag, "SCIENCE_PMOS"    ) == 0 ||
-                strcmp(tag, SCIENCE_IMG       ) == 0 ) {
+                strcmp(tag, "SCIENCE_MOS"     ) == 0 ||
+                strcmp(tag, "SCIENCE_MXU"     ) == 0 ||
+                strcmp(tag, "SCIENCE_LSS"     ) == 0 ||
+                strcmp(tag, "STANDARD_MOS"    ) == 0 ||
+                strcmp(tag, "STANDARD_MXU"    ) == 0 ||
+                strcmp(tag, "STANDARD_LSS"    ) == 0 ||
+                strcmp(tag, SCIENCE_IMG       ) == 0 ||
+                strcmp(tag, "SCREEN_FLAT_MXU" ) == 0 ||
+                strcmp(tag, "SCREEN_FLAT_MOS" ) == 0 ||
+                strcmp(tag, "SCREEN_FLAT_LSS" ) == 0 ) {
                 cpl_frame_set_group(f, CPL_FRAME_GROUP_RAW);
             }
             else if (strcmp(tag, MASTER_BIAS                    ) == 0 ||
@@ -327,6 +297,18 @@ fors_dfs_set_groups(cpl_frameset * set)
                      strcmp(tag, "DISP_COEFF_PMOS"              ) == 0 ||
                      strcmp(tag, "CURV_COEFF_PMOS"              ) == 0 ||
                      strcmp(tag, "SLIT_LOCATION_PMOS"           ) == 0 ||
+                     strcmp(tag, "MASTER_NORM_FLAT_MOS"         ) == 0 ||
+                     strcmp(tag, "MASTER_NORM_FLAT_MXU"         ) == 0 ||
+                     strcmp(tag, "MASTER_NORM_FLAT_LSS"         ) == 0 ||
+                     strcmp(tag, "SLIT_LOCATION_MOS"            ) == 0 ||
+                     strcmp(tag, "SLIT_LOCATION_MXU"            ) == 0 ||
+                     strcmp(tag, "SLIT_LOCATION_LSS"            ) == 0 ||
+                     strcmp(tag, "CURV_COEFF_MOS"               ) == 0 ||
+                     strcmp(tag, "CURV_COEFF_MXU"               ) == 0 ||
+                     strcmp(tag, "CURV_COEFF_LSS"               ) == 0 ||
+                     strcmp(tag, "DISP_COEFF_MOS"               ) == 0 ||
+                     strcmp(tag, "DISP_COEFF_MXU"               ) == 0 ||
+                     strcmp(tag, "DISP_COEFF_LSS"               ) == 0 ||
                      /* static calibration */
                      strcmp(tag, FLX_STD_IMG                    ) == 0 ||
                      strcmp(tag, "MASTER_LINECAT"               ) == 0 ||
@@ -1054,7 +1036,7 @@ dfs_save(cpl_frameset *frameset, const void *object, fors_type type,
 
     cpl_frame_set_filename(frame, filename);
     cpl_frame_set_tag(frame, category);
-    cpl_frame_set_type(frame, type);
+    cpl_frame_set_type(frame, CPL_FRAME_TYPE_ANY);
     cpl_frame_set_group(frame, CPL_FRAME_GROUP_PRODUCT);
     cpl_frame_set_level(frame, CPL_FRAME_LEVEL_FINAL);
     if (cpl_error_get_code()) {
@@ -1093,16 +1075,6 @@ dfs_save(cpl_frameset *frameset, const void *object, fors_type type,
 
     cpl_free((void *)version);
 
-/* CPL3.0 
-    cpl_propertylist_erase_regexp(plist, "^ESO DET OUT1 OVSC*", 0);
-    cpl_propertylist_erase_regexp(plist, "^ESO DET OUT1 PRSC*", 0);
-*/
-/* CPL2.0 */
-    cpl_propertylist_erase(plist, "ESO DET OUT1 OVSCX");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 PRSCX");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 OVSCY");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 PRSCY");
-
     /*
      * Write to disk
      */
@@ -1174,6 +1146,59 @@ fors_dfs_save_image(cpl_frameset *frameset, const cpl_image *image,
              inherit_frame);
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief
+ *    Save DFS product (image)
+ *
+ * @param frameset   see dfs_save_image()
+ * @param image      image (containing data and error bars) to save
+ * @param mask       bad pixel mask with bad pixel codes
+ * @param category   see dfs_save_image()
+ * @param header     see dfs_save_image()
+ * @param parlist    see dfs_save_image()
+ * @param recipename see dfs_save_image()
+ * @param raw_frame  any raw frame used for this product
+ *
+ * This function is the same as dfs_save_image(), except the image is saved using
+ * fors_image_save().
+ *
+ * The instrument version number is determined from the header of the
+ * input raw frame.
+ * 
+ */
+
+/*----------------------------------------------------------------------------*/
+void
+fors_dfs_save_image_mask(cpl_frameset *frameset, const cpl_image *image,
+                         const cpl_image *mask, const char *category, 
+                         cpl_propertylist *header, 
+                         const cpl_parameterlist *parlist, const char *recipename,
+                         const cpl_frame *inherit_frame)
+{
+    char             *filename;
+    cpl_propertylist * extension_header;
+
+    dfs_save(frameset, image, FORS_TYPE_IMAGE, 
+             category, header, 
+             parlist, recipename, 
+             inherit_frame);
+    
+    extension_header = cpl_propertylist_new();
+    cpl_propertylist_append_string(extension_header, 
+                                   "EXTNAME", "IMAGE.BPM");
+
+    filename = cpl_calloc(strlen(category) + 6, sizeof(char));
+
+    strlower(strcpy(filename, category));
+    strcat(filename, ".fits");
+
+    cpl_image_save(mask, filename, CPL_BPP_IEEE_FLOAT, extension_header,
+                   CPL_IO_EXTEND);
+    cpl_propertylist_delete(extension_header);
+
+}
+
 /*----------------------------------------------------------------------------*/
 /**
  * @brief
@@ -1208,6 +1233,59 @@ fors_dfs_save_image_err(cpl_frameset *frameset, const fors_image *image,
              inherit_frame);
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+ * @brief
+ *    Save DFS product (image) with it error data and a bad pixel mask
+ *
+ * @param frameset   see dfs_save_image()
+ * @param image      image (containing data and error bars) to save
+ * @param mask       bad pixel mask with bad pixel codes
+ * @param category   see dfs_save_image()
+ * @param header     see dfs_save_image()
+ * @param parlist    see dfs_save_image()
+ * @param recipename see dfs_save_image()
+ * @param raw_frame  any raw frame used for this product
+ *
+ * This function is the same as dfs_save_image(), except the image is saved using
+ * fors_image_save().
+ *
+ * The instrument version number is determined from the header of the
+ * input raw frame.
+ * 
+ * TODO: Use dfs_save for all the stuff
+ */
+
+/*----------------------------------------------------------------------------*/
+void
+fors_dfs_save_image_err_mask(cpl_frameset *frameset, const fors_image *image,
+                             const cpl_image *mask, const char *category,
+                             cpl_propertylist *header,
+                             const cpl_parameterlist *parlist, const char *recipename,
+                             const cpl_frame *inherit_frame)
+{
+    char             *filename;
+    cpl_propertylist * extension_header;
+        
+    dfs_save(frameset, image, FORS_TYPE_IMAGE_ERR, 
+             category, header, 
+             parlist, recipename, 
+             inherit_frame);
+    
+    extension_header = cpl_propertylist_new();
+    cpl_propertylist_append_string(extension_header, 
+                                   "EXTNAME", "IMAGE.BPM");
+
+    filename = cpl_calloc(strlen(category) + 6, sizeof(char));
+
+    strlower(strcpy(filename, category));
+    strcat(filename, ".fits");
+
+    cpl_image_save(mask, filename, CPL_BPP_IEEE_FLOAT, extension_header,
+                   CPL_IO_EXTEND);
+    cpl_propertylist_delete(extension_header);
+}
+
 #undef cleanup
 #define cleanup \
 do { \
@@ -1408,13 +1486,8 @@ int dfs_save_image(cpl_frameset *frameset, const cpl_image *image,
     else
         plist = header;
 
-#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 8, 0)
     if (cpl_dfs_setup_product_header(plist, frame, frameset, parlist,
                                      recipename, version, "PRO-1.15", NULL)) {
-#else 
-    if (cpl_dfs_setup_product_header(plist, frame, frameset, parlist,
-                                     recipename, version, "PRO-1.15")) {
-#endif
         cpl_msg_error(cpl_func, "Error found in %s: %s",
                       cpl_error_get_where(), cpl_error_get_message());
         cpl_msg_error(func, "Problem with product %s FITS header definition",
@@ -1426,16 +1499,6 @@ int dfs_save_image(cpl_frameset *frameset, const cpl_image *image,
         return -1;
     }
 
-/* CPL3.0 
-    cpl_propertylist_erase_regexp(plist, "^ESO DET OUT1 OVSC*", 0);
-    cpl_propertylist_erase_regexp(plist, "^ESO DET OUT1 PRSC*", 0);
-*/
-/* CPL2.0 */
-    cpl_propertylist_erase(plist, "ESO DET OUT1 OVSCX");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 PRSCX");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 OVSCY");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 PRSCY");
-
     /*
      * Write image to disk
      */
@@ -1516,16 +1579,10 @@ int dfs_save_table(cpl_frameset *frameset, const cpl_table *table,
 
     cpl_msg_info(func, "Saving %s table to disk...", category);
 
-    /*
-    filename = cpl_calloc(strlen(category) + 7, sizeof(char));
-    */
     filename = cpl_calloc(strlen(category) + 6, sizeof(char));
 
     strlower(strcpy(filename, category));
 
-    /*
-    strcat(filename, ".tfits");
-    */
     strcat(filename, ".fits");
 
     frame = cpl_frame_new();
@@ -1554,13 +1611,8 @@ int dfs_save_table(cpl_frameset *frameset, const cpl_table *table,
     else
         plist = header;
 
-#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 8, 0)
     if (cpl_dfs_setup_product_header(plist, frame, frameset, parlist,
                                      recipename, version, "PRO-1.15", NULL)) {
-#else 
-    if (cpl_dfs_setup_product_header(plist, frame, frameset, parlist,
-                                     recipename, version, "PRO-1.15")) {
-#endif
         cpl_msg_error(cpl_func, "Error found in %s: %s",
                       cpl_error_get_where(), cpl_error_get_message());
         cpl_msg_error(func, "Problem with product %s FITS header definition",
@@ -1572,21 +1624,10 @@ int dfs_save_table(cpl_frameset *frameset, const cpl_table *table,
         return -1;
     }
 
-/* For CPL3.0
-    cpl_propertylist_erase_regexp(plist, "^ESO DET OUT1 OVSC*", 0);
-    cpl_propertylist_erase_regexp(plist, "^ESO DET OUT1 PRSC*", 0);
-*/
-/* For CPL2.0 */
-    cpl_propertylist_erase(plist, "ESO DET OUT1 OVSCX");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 PRSCX");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 OVSCY");
-    cpl_propertylist_erase(plist, "ESO DET OUT1 PRSCY");
-
     /*
      * Write table to disk
      */
     
-    
     if (cpl_table_save(table, plist, NULL, filename, CPL_IO_DEFAULT)) {
         cpl_msg_error(cpl_func, "Error found in %s: %s",
                       cpl_error_get_where(), cpl_error_get_message());
@@ -1841,10 +1882,6 @@ cpl_error_code dfs_save_image_null(cpl_frameset * frameset,
 				   const char * recipename,
 				   const char * version)
 {
-    const char * regexp = "ESO DET OUT1 OVSCX|"
-	                  "ESO DET OUT1 PRSCX|"
-                          "ESO DET OUT1 OVSCY|"
-                          "ESO DET OUT1 PRSCY";
 
     char * filename = cpl_calloc(strlen(tag) + 6, sizeof(char));
 
@@ -1857,15 +1894,9 @@ cpl_error_code dfs_save_image_null(cpl_frameset * frameset,
     strlower(strcpy(filename, tag));
     strcat(filename, ".fits");
 
-#if defined CPL_VERSION_CODE && CPL_VERSION_CODE >= CPL_VERSION(4, 8, 0)
     error = cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL, NULL,
 			       CPL_BPP_IEEE_FLOAT, recipename, pro,
-			       regexp, version, filename);
-#else
-    error = cpl_dfs_save_image(frameset, parlist, frameset, NULL,
-			       CPL_BPP_IEEE_FLOAT, recipename, tag,
-			       NULL, regexp, version, filename);
-#endif
+			       NULL, version, filename);
     
     cpl_free(filename);
     cpl_propertylist_delete(pro);
@@ -1873,44 +1904,5 @@ cpl_error_code dfs_save_image_null(cpl_frameset * frameset,
     return error;
 }
 
-cpl_frameset * hawki_extract_frameset(
-        const cpl_frameset  *   in,
-        const char          *   tag)
-{
-    cpl_frameset    *   out ;
-    const cpl_frame *   cur_frame ;
-    cpl_frame       *   loc_frame ;
-    int                 nbframes, nbext ;
-    int                 i ;
-
-    /* Test entries */
-    if (in == NULL) return NULL ;
-    if (tag == NULL) return NULL ;
-
-    /* Initialise */
-    nbframes = cpl_frameset_get_size(in) ;
-
-    /* Count the frames with the tag */
-    if ((nbext = cpl_frameset_count_tags(in, tag)) == 0) return NULL ;
-
-    /* Create the output frameset */
-    out = cpl_frameset_new() ;
-
-    /* Loop on the requested frames and store them in out */
-    nbext = 0 ;
-    for (i=0 ; i<nbframes ; i++) {
-        cur_frame = cpl_frameset_get_frame_const(in, i) ;
-        if (!strcmp(cpl_frame_get_tag(cur_frame), tag)) {
-            loc_frame = cpl_frame_duplicate(cur_frame) ;
-            cpl_frameset_insert(out, loc_frame) ;
-            nbext ++ ;
-        }
-    }
-    return out ;
-}
-
-
-
-
 
 /**@}*/
