@@ -30,13 +30,25 @@
 #endif
 
 #include <cmath>
+#include <functional>
 #include <fors_flat_normalise.h>
 #include <moses.h>
 #include <image_smooth.h>
+#include <vector_utils.h>
 #include <image_spline_fit.h>
+#include <image_normalisation.h>
 
 #define STRETCH_FACTOR   (1.20)
 
+
+fors::flat_normaliser::flat_normaliser() : m_normalisation_image(NULL)
+{
+}
+
+fors::flat_normaliser::~flat_normaliser()
+{
+}
+    
 /**
  * @brief 
  *   Normalise a flat field exposure
@@ -67,15 +79,15 @@
  * will consist of a polynomial fitting of the illumination profile along
  * the dispersion direction, performed independently for each row of the
  * spatially remapped spectra.
- */
- 
-cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial, 
-                                 cpl_table *slits, cpl_table *polytraces, 
-                                 double reference, double blue, double red, 
-                                 double dispersion,
-                                 int spa_smooth_radius, int disp_smooth_radius, 
-                                 int spa_fit_nknots, int disp_fit_nknots,
-                                 double fit_threshold)
+ */ 
+int fors::flat_normaliser::mos_normalise
+(mosca::image& flat, cpl_image *spatial, 
+ cpl_table *slits, cpl_table *polytraces, 
+ double reference, double blue, double red, 
+ double dispersion,
+ int spa_smooth_radius, int disp_smooth_radius, 
+ int spa_fit_polyorder, int disp_fit_nknots,
+ double fit_threshold)
 {
     const char     *func = "mos_mosflat_normalise";
 
@@ -84,7 +96,6 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
 
     cpl_image      *rectified;
     cpl_image      *smo_flat;
-    cpl_image      *exslit;
     cpl_polynomial *polytop;
     cpl_polynomial *polybot;
 
@@ -114,26 +125,29 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
     int             i, j;
     cpl_size        k;
 
-/*    int             exclude = 5;     Number of excluded pixels at edges */
 
     /* For debug puposes only: cpl_image      *smo_rectified; */
 
-    if (flat == NULL || slits == NULL || polytraces == NULL) {
+    if (flat.get_cpl_image() == NULL || slits == NULL || polytraces == NULL) {
         cpl_error_set(func, CPL_ERROR_NULL_INPUT);
-        return NULL;
+        return 1;
     }
  
     if (dispersion <= 0.0) {
         cpl_error_set(func, CPL_ERROR_ILLEGAL_INPUT);
-        return NULL;
+        return 1;
     }
 
     if (red - blue < dispersion) {
         cpl_error_set(func, CPL_ERROR_ILLEGAL_INPUT);
-        return NULL;
+        return 1;
     }
     
-    rectified = mos_spatial_calibration(flat, slits, polytraces, reference,
+    cpl_image * flat_im = flat.get_cpl_image();
+    cpl_image * flat_err = flat.get_cpl_image_err();
+
+    
+    rectified = mos_spatial_calibration(flat_im, slits, polytraces, reference,
                                         blue, red, dispersion, 0, NULL);
 
     nx = cpl_image_get_size_x(rectified);
@@ -159,6 +173,7 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
 
     xlow = 1;
     xhig = nx;
+    m_wave_profiles.clear();
     for (i = 0; i < nslits; i++) {
 
         if (length[i] == 0)
@@ -177,41 +192,38 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
         ylow = position[i] + 1;
         yhig = ylow + length[i] - 1;
 
-        exslit = cpl_image_extract(rectified, xlow, ylow, xhig, yhig);
-        mosca::image im_slit(exslit);
-
-        if (spa_smooth_radius > 0) 
-        {
-            int  final_spa_smooth_radius;
-            if (im_slit.size_spatial() / 2 < spa_smooth_radius)
-            {
-                final_spa_smooth_radius = im_slit.size_spatial() / 2;
-                cpl_msg_warning(cpl_func, "Slit too narrow for requested "
-                                "smoothing radius %d. Using %d", 
-                                spa_smooth_radius, final_spa_smooth_radius);
-            }
-            else
-                final_spa_smooth_radius = spa_smooth_radius;
-            mosca::image_smooth_1d_median<float>(im_slit, 
-                                                 final_spa_smooth_radius, 
-                                                 mosca::SPATIAL_AXIS);
-        }
+        cpl_image * exslit_orig =
+                cpl_image_extract(rectified, xlow, ylow, xhig, yhig);
+        mosca::image im_exslit_orig(exslit_orig, true);
         
-        if (spa_fit_nknots > 0)
-            mosca::image_cubicspline_1d_fit<float>(im_slit, spa_fit_nknots, 
-                                                   fit_threshold,
-                                                   mosca::SPATIAL_AXIS);
+        int  final_spa_smooth_radius = spa_smooth_radius;
+        if (im_exslit_orig.size_spatial() / 2 < spa_smooth_radius)
+        {
+            final_spa_smooth_radius = im_exslit_orig.size_spatial() / 2;
+            cpl_msg_warning(cpl_func, "Slit too narrow for requested "
+                            "smoothing radius %d. Using %d", 
+                            spa_smooth_radius, final_spa_smooth_radius);
+        }
 
-        if (disp_smooth_radius > 0)
-            mosca::image_smooth_1d_median<float>(im_slit, 
-                                                 disp_smooth_radius, 
-                                                 mosca::DISPERSION_AXIS);
-        if (disp_fit_nknots > 0)
-            mosca::image_cubicspline_1d_fit<float>(im_slit, disp_fit_nknots, 
-                                                   fit_threshold,
-                                                   mosca::DISPERSION_AXIS);
-        //image_smooth_fit_1d_pol_disp(exslit, disp_fit_polyorder);
+        
+        std::vector<float> slit_spa_norm_profile;
+        std::vector<float> slit_disp_norm_profile;
+        mosca::image normslit = mosca::image_normalise<float>(im_exslit_orig, 
+                                      final_spa_smooth_radius, disp_smooth_radius,
+                                      spa_fit_polyorder, disp_fit_nknots, 
+                                      fit_threshold, slit_spa_norm_profile,
+                                      slit_disp_norm_profile);
+        float mean_disp_profile_flux = 
+            std::accumulate(slit_disp_norm_profile.begin(), 
+                            slit_disp_norm_profile.end(), 0.);
+        mean_disp_profile_flux /= slit_disp_norm_profile.size();
+        
+        for(size_t i_prof = 0; i_prof < slit_disp_norm_profile.size(); i_prof++)
+            slit_disp_norm_profile[i_prof] /= mean_disp_profile_flux;
 
+        m_wave_profiles.push_back(slit_disp_norm_profile);
+        
+        cpl_image * exslit = normslit.get_cpl_image();
         
         /*
          * Recover from the table of spectral curvature coefficients
@@ -298,8 +310,8 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
          * (compare with function mos_spatial_calibration()).
          */
 
-        nx = cpl_image_get_size_x(flat);
-        ny = cpl_image_get_size_y(flat);
+        nx = cpl_image_get_size_x(flat_im);
+        ny = cpl_image_get_size_y(flat_im);
 
         sdata = cpl_image_get_data_float(spatial);
         xdata = cpl_image_get_data_float(exslit);
@@ -362,7 +374,6 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
         }
         cpl_polynomial_delete(polytop);
         cpl_polynomial_delete(polybot);
-        cpl_image_delete(exslit);
     }
     
     //TODO: What happens with inter-slit flux conservation. In principle the overall level
@@ -370,10 +381,16 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
 
     cpl_image_delete(rectified);
 
-    cpl_image_divide(flat, smo_flat);
+    cpl_image_divide(flat_im, smo_flat);
+    cpl_image_divide(flat_err, smo_flat);
 
-    return smo_flat;
+    
+    //TODO: This is giving some segmentation fault. Fix it 
+    //m_normalisation_image = mosca::image(smo_flat);
+    
+    return 0;
 }
+    
 
 
 /**
@@ -398,221 +415,86 @@ cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial,
  * consist of a polynomial fitting along the requested direction performed 
  * independently for each column/row of the image.
  */
-cpl_image *mos_lssflat_normalise
-(cpl_image *flat, int spa_smooth_radius, int disp_smooth_radius, 
- int spa_fit_nknots, int disp_fit_nknots,
+int fors::flat_normaliser::lss_normalise
+(mosca::image& flat,
+ int spa_smooth_radius, int disp_smooth_radius, 
+ int spa_fit_polyorder, int disp_fit_nknots,
  double fit_threshold)
 {
     const char     *func = "mos_normalise_longflat";
 
     cpl_image      *smo_flat;
 
-    if (flat == NULL) {
+    if (flat.get_cpl_image() == NULL) {
         cpl_error_set(func, CPL_ERROR_NULL_INPUT);
-        return NULL;
+        return 1;
     }
  
-    if (spa_fit_nknots < 1 && spa_smooth_radius < 1) {
-        cpl_error_set(func, CPL_ERROR_ILLEGAL_INPUT);
-        return NULL;
-    }
-    if (disp_fit_nknots < 1 && disp_smooth_radius < 1) {
-        cpl_error_set(func, CPL_ERROR_ILLEGAL_INPUT);
-        return NULL;
-    }
+    cpl_image * flat_im = flat.get_cpl_image();
+    cpl_image * flat_err = flat.get_cpl_image_err();
 
-    smo_flat = cpl_image_duplicate(flat);
-    mosca::image smoothed_flat(smo_flat);
+    smo_flat = cpl_image_duplicate(flat_im);
+    mosca::image smoothed_flat(smo_flat, true);
 
-    if (spa_fit_nknots < 0)
+    int  final_spa_smooth_radius = spa_smooth_radius;
+    if (smoothed_flat.size_spatial() / 2 < spa_smooth_radius)
     {
-        mosca::image_smooth_1d_median<float>(smoothed_flat, 
-                                             spa_smooth_radius, 
-                                             mosca::SPATIAL_AXIS);        
+        final_spa_smooth_radius = smoothed_flat.size_spatial() / 2;
+        cpl_msg_warning(cpl_func, "Slit too narrow for requested "
+                        "smoothing radius %d. Using %d", 
+                        spa_smooth_radius, final_spa_smooth_radius);
     }
-    else
-        mosca::image_cubicspline_1d_fit<float>(smoothed_flat, spa_fit_nknots, 
-                                               fit_threshold,
-                                               mosca::SPATIAL_AXIS);
-        //image_smooth_fit_1d_pol_spa(smo_flat, spa_fit_polyorder);
 
-    if (disp_fit_nknots < 0) 
+    std::vector<float> slit_spa_norm_profile;
+    std::vector<float> slit_disp_norm_profile;
+    mosca::image normslit = mosca::image_normalise<float>(smoothed_flat, 
+                                  final_spa_smooth_radius, disp_smooth_radius,
+                                  spa_fit_polyorder, disp_fit_nknots, 
+                                  fit_threshold, slit_spa_norm_profile,
+                                  slit_disp_norm_profile);
+    float mean_disp_profile_flux = 
+        std::accumulate(slit_disp_norm_profile.begin(), 
+                        slit_disp_norm_profile.end(), 0.);
+    mean_disp_profile_flux /= slit_disp_norm_profile.size();
+    
+    for(size_t i_prof = 0; i_prof < slit_disp_norm_profile.size(); i_prof++)
+        slit_disp_norm_profile[i_prof] /= mean_disp_profile_flux;
+
+    m_wave_profiles.push_back(slit_disp_norm_profile);
+
+
+    //Finally divide by the smoothed version of the flat.
+    cpl_image_divide(flat_im, normslit.get_cpl_image());
+    cpl_image_divide(flat_err, normslit.get_cpl_image());
+
+    //TODO: This is giving some segmentation fault. Fix it 
+    //m_normalisation_image = normslit;
+    
+    return 0;
+}
+
+const mosca::image& fors::flat_normaliser::get_normalisation_image() const
+{
+    return m_normalisation_image;
+}
+    
+const std::vector<std::vector<float> >& fors::flat_normaliser::get_wave_profiles() const
+{
+    return m_wave_profiles;
+}
+
+cpl_image * fors::flat_normaliser::get_wave_profiles_im() const
+{
+    cpl_image * wave_profiles_im = 
+            cpl_image_new(m_wave_profiles.front().size(), m_wave_profiles.size(), 
+                          CPL_TYPE_FLOAT);
+    float * wave_profiles_data = cpl_image_get_data_float(wave_profiles_im);
+    for(size_t i_slit = 0 ; i_slit < m_wave_profiles.size(); ++i_slit)
     {
-        mosca::image_smooth_1d_median<float>(smoothed_flat, 
-                                             disp_smooth_radius, 
-                                             mosca::DISPERSION_AXIS);
-    }
-    else 
-        mosca::image_cubicspline_1d_fit<float>(smoothed_flat, disp_fit_nknots, 
-                                               fit_threshold,
-                                               mosca::DISPERSION_AXIS);
-        //image_smooth_fit_1d_pol_disp(smo_flat, disp_fit_polyorder);
-
-    cpl_image_divide(flat, smo_flat);
-
-    return smo_flat;
-}
-
-/**
- * Smooth an image fitting a polynomial in spatial direction 
- * @param image       Image to be smoothed
- * @param polyorder   the order of the polynomial to fit
- * 
- * This function will smooth a MOS image in the spatial direction 
- * (here assumed to be the Y axis) fitting each column with a polynomial. 
- */
-void image_smooth_fit_1d_pol_spa(cpl_image * image, cpl_size polyorder)
-{
-    /*
-     * Fit with a polynomial the flat field trend column by column.
-     */
-
-    cpl_image_turn(image, -1);   /* For faster memory access */
-
-    cpl_size nx  = cpl_image_get_size_x(image);
-    cpl_size ny  = cpl_image_get_size_y(image);
-    float * data = cpl_image_get_data_float(image);
-
-    cpl_image * profile = cpl_image_collapse_median_create(image, 1, 0, 0);
-    float * level = cpl_image_get_data_float(profile);
-
-    for (cpl_size i = 0; i < ny; i++) {
-
-        /*
-         * First get the size of the vectors to allocate:
-         * eliminate from fit any value more than 20% away
-         * from median level in current column.
-         */
-
-        cpl_size npoints = 0;
-        float * p = data + i*nx;
-        for (cpl_size j = 0; j < nx; j++)
-            if (std::fabs(p[j]/level[i] - 1) < 0.20)
-                npoints++;
-
-        if (npoints > polyorder + 1) {
-
-            /*
-             * Fill the vectors for the fitting
-             */
-
-            cpl_vector * flux = cpl_vector_new(npoints);
-            double * fdata = cpl_vector_get_data(flux);
-            cpl_vector * positions = cpl_vector_new(npoints);
-            double * pdata = cpl_vector_get_data(positions);
-
-            npoints = 0;
-            p = data + i*nx;
-            for (cpl_size j = 0; j < nx; j++) {
-                if (fabs(p[j]/level[i] - 1) < 0.20) {
-                    fdata[npoints] = p[j];
-                    pdata[npoints] = j;
-                    npoints++;
-                }
-            }
-
-            cpl_polynomial * trend = cpl_polynomial_fit_1d_create
-                    (positions, flux, polyorder, NULL);
-
-            cpl_vector_delete(flux);
-            cpl_vector_delete(positions);
-
-            if (trend) {
-                p = data + i*nx;
-                for (cpl_size j = 0; j < nx; j++)
-                    p[j] = cpl_polynomial_eval_1d(trend, j, NULL);
-                cpl_polynomial_delete(trend);
-            }
-            else {
-                cpl_msg_warning(cpl_func, 
-                                "Invalid flat field flux fit (ignored)");
-            }
-        }
-    }
-
-    cpl_image_delete(profile);
-    cpl_image_turn(image, 1);
-}
-
-/**
- * Smooth an image fitting a polynomial in dispersion direction 
- * @param image       Image to be smoothed
- * @param polyorder   the order of the polynomial to fit
- * 
- * This function will smooth a MOS image in the dispersion direction 
- * (here assumed to be the X axis) fitting each row with a polynomial.
- * TODO: merge this with image_smooth_fit_1d_pol_spa() and place it in MOSCA.  
- */
-void image_smooth_fit_1d_pol_disp(cpl_image * image, cpl_size polyorder)
-{
-    float          *p;
-    float          *data;
-    double         *fdata;
-    double         *pdata;
-    int             npoints;
-
-    cpl_vector     *positions;
-    cpl_vector     *flux;
-    cpl_polynomial *trend;
-
-    /*
-     * Fit with a polynomial the flat field trend row by row.
-     */
-
-    cpl_size nx = cpl_image_get_size_x(image);
-    cpl_size ny = cpl_image_get_size_y(image);
-    data = cpl_image_get_data_float(image);
-
-    for (cpl_size j = 0; j < ny; j++) {
-
-        /*
-         * First get the size of the vectors to allocate:
-         */
-
-        npoints = 0;
-        p = data + j*nx;
-        for (cpl_size k = 0; k < nx; k++)
-            if (p[k] > 1.0)
-                npoints++;
-
-        if (npoints > polyorder + 1) {
-
-            /*
-             * Fill the vectors for the fitting
-             */
-
-            flux = cpl_vector_new(npoints);
-            fdata = cpl_vector_get_data(flux);
-            positions = cpl_vector_new(npoints);
-            pdata = cpl_vector_get_data(positions);
-
-            npoints = 0;
-            p = data + j*nx;
-            for (cpl_size k = 0; k < nx; k++) {
-                if (p[k] > 1.0) {
-                    fdata[npoints] = p[k];
-                    pdata[npoints] = k;
-                    npoints++;
-                }
-            }
-
-            trend = cpl_polynomial_fit_1d_create(positions, flux, 
-                                                 polyorder, NULL);
-
-            cpl_vector_delete(flux);
-            cpl_vector_delete(positions);
-
-            if (trend) {
-                p = data + j*nx;
-                for (cpl_size k = 0; k < nx; k++)
-                    if (p[k] > 1.0)
-                        p[k] = cpl_polynomial_eval_1d(trend, k, NULL);
-                cpl_polynomial_delete(trend);
-            }
-            else {
-                cpl_msg_warning(cpl_func, "Invalid flat field flux fit "
-                                "(ignored)");
-            }
-        }
+        wave_profiles_data = 
+            std::copy(m_wave_profiles[i_slit].begin(), 
+                      m_wave_profiles[i_slit].end(), wave_profiles_data);
     }
+    
+    return wave_profiles_im;
 }
