@@ -29,24 +29,47 @@
 #define FORS_FLAT_NORMALISE_H
 
 #include <cpl.h>
+#include "mosca_image.h"
 
-CPL_BEGIN_DECLS
+namespace fors
+{
 
-cpl_image *mos_mosflat_normalise(cpl_image *flat, cpl_image *spatial, 
-                                 cpl_table *slits, cpl_table *polytraces, 
-                                 double reference, double blue, double red, 
-                                 double dispersion,
-                                 int spa_smooth_radius, int disp_smooth_radius, 
-                                 int spa_fit_nknots, int disp_fit_nknots,
-                                 double fit_threshold);
-cpl_image *mos_lssflat_normalise(cpl_image *flat,
-                                 int spa_smooth_radius, int disp_smooth_radius, 
-                                 int spa_fit_nknots, int disp_fit_nknots,
-                                 double fit_threshold);
+class flat_normaliser
+{
+    
+public:
 
-void image_smooth_fit_1d_pol_spa(cpl_image * image, cpl_size polyorder);
-void image_smooth_fit_1d_pol_disp(cpl_image * image, cpl_size polyorder);
+    flat_normaliser();
+    ~flat_normaliser();
+    
+    int mos_normalise(mosca::image& flat, cpl_image *spatial, 
+                      cpl_table *slits, cpl_table *polytraces, 
+                      double reference, double blue, double red, 
+                      double dispersion,
+                      int spa_smooth_radius, int disp_smooth_radius, 
+                      int spa_fit_polyorder, int disp_fit_nknots,
+                      double fit_threshold);
+    
+    int lss_normalise(mosca::image& flat,
+                       int spa_smooth_radius, int disp_smooth_radius, 
+                       int spa_fit_polyorder, int disp_fit_nknots,
+                       double fit_threshold);
 
-CPL_END_DECLS
+    const mosca::image& get_normalisation_image() const;
+    
+    const std::vector<std::vector<float> >& get_wave_profiles() const;
+    
+    cpl_image * get_wave_profiles_im() const;
+
+private:
+    
+    mosca::image m_normalisation_image;
+    
+    std::vector<std::vector<float> > m_wave_profiles;
+    
+    
+};
+
+}
 
 #endif   /* FORS_FLAT_NORMALISE_H */
