@@ -35,7 +35,8 @@
 #include <fors_utils.h>
 #include <fors_pfits.h>
 #include <fors_double.h>
-
+#include <fors_saturation.h>
+#include <fors_subtract_bias.h>
 #include <moses.h>
 
 #include <cpl.h>
@@ -44,33 +45,6 @@
 #include <stdbool.h>
 #include <stdio.h>
 
-/**
- * @defgroup fors_image Image with error bars
- *
- * This object contains 
- * - the image data values,
- * - the image data error bars,
- * - additional relevant properties for the image 
- *   (detector overscan areas, exposure time, etc.)
- *
- * For efficiency+convenience the error bars are internally represented
- * as sigma^2 to avoid the repeated conversion between sigma <-> variance
- */
-
-/**@{*/
-
-struct _fors_image
-{
-    cpl_image *data;
-    cpl_image *variance;
-
-    /* Invariants:
-       The CPL images are non-NULL. 
-       The variance image is everywhere non-negative.
-       The CPL image types are FORS_IMAGE_TYPE.
-       The CPL image bad pixel masks are unused
-    */
-};
 
 /**
  * @brief    Image data type
@@ -257,9 +231,7 @@ do { \
  *
  */
 fors_image_list *
-fors_image_load_list(const cpl_frameset *frames, const fors_image *bias,
-                     const fors_setting *setting,
-                     double *saturated)
+fors_image_load_list(const cpl_frameset *frames)
 {
     fors_image_list *ilist = fors_image_list_new();
     double_list *sat_percent = double_list_new();
@@ -270,33 +242,16 @@ fors_image_load_list(const cpl_frameset *frames, const fors_image *bias,
     {
         const cpl_frame *f;
         
-        for (f = cpl_frameset_get_first_const(frames);
-             f != NULL;
-             f = cpl_frameset_get_next_const(frames)) {
+        for (int i =0; i< cpl_frameset_get_size(frames); i ++) 
+        {
+            f = cpl_frameset_get_position_const(frames, i);
+            
+            fors_image *ima = fors_image_load(f);
 
-            double saturated_one;
-            
-            fors_image *i = fors_image_load(f, bias, setting,
-                                            saturated != NULL ? 
-                                            &saturated_one : NULL);
-            assure( !cpl_error_get_code(), return ilist, NULL );
-            
-            fors_image_list_insert(ilist, i);
-            if (saturated != NULL) {
-                double_list_insert(sat_percent, 
-                                   double_duplicate(&saturated_one));
-            }
+            fors_image_list_insert(ilist, ima);
         }
     }
 
-    /* fixme: make sure input is consistent */
-
-    /* Compute overall percentage as mean of each percentage.
-       Valid because input frames have same size. */
-    if (saturated != NULL) {
-        *saturated = double_list_mean(sat_percent, double_eval, NULL);
-    }
-
     cleanup;
     return ilist;
 }
@@ -313,64 +268,37 @@ fors_image_load_list(const cpl_frameset *frames, const fors_image *bias,
  * an array of const images.
  */
 const fors_image_list *
-fors_image_load_list_const(const cpl_frameset *frames, const fors_image *bias,
-                           const fors_setting *setting,
-                           double *saturated)
+fors_image_load_list_const(const cpl_frameset *frames)
 {
     return (const fors_image_list *)
-        fors_image_load_list(frames, bias, setting, saturated);
+        fors_image_load_list(frames);
 }
 
 #undef cleanup
 #define cleanup \
 do { \
-    cpl_propertylist_delete(header); \
-    cpl_table_delete(overscans); \
     cpl_image_delete(temp); \
-    cpl_mask_delete(non_saturated); \
-    fors_setting_delete(&frame_setting); \
 } while (0)
 /**
  * @brief    Load image
  * @param    frame         to load
- * @param    bias          master bias image, or NULL
- * @param    setting       expected instrument setting
- * @param    saturated     (output) If non-NULL, percentage of saturated pixels in
-                           the raw frame (before bias subtraction),
-                           defined as > 65534.4 or < 0.5
- * @return   Newly allocated bias image, or bias-subtracted image
+ * @return   Newly allocated image
  *
- * The provided bias image is subtracted and used to define the raw frame
- * error bars. If NULL, the input frame is assumed to be a raw bias
- * and the error bars are given by the read-out-noise
- * as defined in the FITS header. If non-NULL, this bias image
- * is subtracted and the residual flux is used to estimate the photonic noise.
- *
- * This function currently does not do overscan area correction.
+ * The image is loaded 
  */
 fors_image *
-fors_image_load(const cpl_frame *frame, const fors_image *bias,
-                const fors_setting *setting,
-                double *saturated)
+fors_image_load(const cpl_frame *frame)
 {
     fors_image *image           = NULL;
-    fors_setting *frame_setting = NULL;
     cpl_image *data             = NULL;
     cpl_image *variance         = NULL;
     cpl_image *temp             = NULL;
-    cpl_propertylist *header    = NULL;
-    cpl_table *overscans        = NULL;
-    cpl_mask *non_saturated     = NULL;
     const char *filename;
     int extension= 0;
     const int plane = 0;
-    double ocorr = 0.0;
-    int has_overscans = 0;
-    int i;
 
     assure( frame != NULL, return image, NULL );
     /* bias may be NULL */
-    assure( setting != NULL, return image, NULL );
     filename = cpl_frame_get_filename(frame);
     assure( filename != NULL, return image, 
             "NULL filename received");
@@ -381,19 +309,6 @@ fors_image_load(const cpl_frame *frame, const fors_image *bias,
 		 cpl_frame_get_tag(frame) : "NULL",
                  filename);
 
-    /* Verify instrument setting */
-    fors_setting_verify(setting, frame, &frame_setting);
-    assure( !cpl_error_get_code(), return image, 
-            "Could not verify %s setting", 
-            filename);
-    
-
-    /* Get header */
-    header = cpl_propertylist_load(filename, extension);
-    assure( !cpl_error_get_code(), return image, 
-            "Could not load %s extension %d header", 
-            filename, extension);
-
     /* Get data */
     data = cpl_image_load(filename, 
                           FORS_IMAGE_TYPE, plane, extension);
@@ -402,80 +317,16 @@ fors_image_load(const cpl_frame *frame, const fors_image *bias,
             "Could not load image from %s extension %d", 
             filename, extension);
 
-    /* Read, remove overscan areas */
-    {
-        int xlow, ylow, xhig, yhig;
-	/* bool check_consistency = false; */
 
-        overscans = mos_load_overscans_fors(header);
-	/* Uses always ESO.DET.OUT1.*  which is fine even
-	   for old FORS data */
+    /* Read variance if it exists */
+    if (cpl_frame_get_nextensions(frame) == 0) {
 
-        assure( !cpl_error_get_code(), return image, 
-                "Could not read overscan information from %s extension %d", 
-                filename, extension);
-
-        for (i = 1; i < cpl_table_get_nrow(overscans); i++) {
-
-            /*
-             * Overscan correction
-             */
-        
-            xlow = cpl_table_get_int(overscans, "xlow", i, NULL);
-            ylow = cpl_table_get_int(overscans, "ylow", i, NULL);
-            xhig = cpl_table_get_int(overscans, "xhig", i, NULL);
-            yhig = cpl_table_get_int(overscans, "yhig", i, NULL);
-
-            temp = cpl_image_extract(data, xlow+1, ylow+1, xhig, yhig);
-
-            ocorr += cpl_image_get_median(temp);
-
-            cpl_image_delete(temp); temp = NULL;
-        }
-
-        if (cpl_table_get_nrow(overscans) > 1) {
-            has_overscans = 1;
-            ocorr /= cpl_table_get_nrow(overscans) - 1;
-        }
-
-        xlow = cpl_table_get_int(overscans, "xlow", 0, NULL);
-        ylow = cpl_table_get_int(overscans, "ylow", 0, NULL);
-        xhig = cpl_table_get_int(overscans, "xhig", 0, NULL);
-        yhig = cpl_table_get_int(overscans, "yhig", 0, NULL);
-        
-        cpl_table_delete(overscans); overscans = NULL;
-        
-        temp = cpl_image_duplicate(data);
-        cpl_image_delete(data);
-        data = cpl_image_extract(temp, xlow+1, ylow+1, xhig, yhig);
-        
-        cpl_image_delete(temp); temp = NULL;
-    }
-
-    cpl_propertylist_delete(header); header = NULL;       
-
-    /* Define variance */
-    if (cpl_frame_get_nextensions(frame) == 0 || bias != NULL) {
-
-        /* No error bars provided, assume RON only.
-           If frame is not a bias, add photon noise later.
-        */
-
-        /* Define read-out-noise */
+        /* Create an empty variance */
         variance = cpl_image_new(
             cpl_image_get_size_x(data),
             cpl_image_get_size_y(data),
             FORS_IMAGE_TYPE);        
 
-        if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_CALIB) {
-            cpl_msg_warning(cpl_func, 
-                            "No error bars provided for calibration frame %s, "
-                            "assuming no errors. For complete error propagation, "
-                            "you may recreate this frame with this pipeline", 
-                            filename);
-        } else {
-            cpl_image_add_scalar(variance, frame_setting->ron*frame_setting->ron);
-        }
     }
     else {
 
@@ -499,70 +350,6 @@ fors_image_load(const cpl_frame *frame, const fors_image *bias,
     }
     
     image = fors_image_new(data, variance);
-
-    /* Count saturated pixels */
-    if (saturated != NULL) {
-        double lo_cut = 0.5;
-        double hi_cut = 65534.5;
-        non_saturated = cpl_mask_threshold_image_create(
-            data, lo_cut, hi_cut);
-        
-        *saturated = (cpl_image_get_size_x(data) * 
-                      cpl_image_get_size_y(data) - 
-                      cpl_mask_count(non_saturated)) * 100.0 / 
-            (cpl_image_get_size_x(data) * 
-             cpl_image_get_size_y(data));
-
-        cpl_mask_delete(non_saturated); non_saturated = NULL;
-
-        cpl_msg_debug(cpl_func,
-                     "%f %% saturated pixels", *saturated);
-    }        
-    
-    if (bias == NULL) {
-        /* Done. No bias to subtract */
-    }
-    else {
-        assure( cpl_frame_get_group(frame) == CPL_FRAME_GROUP_RAW,
-                return image,
-                "Refusing to subtract bias from non-raw (%s) input frame: %s",
-                fors_frame_get_group_string(frame),
-                filename);
-        
-        cpl_msg_debug(cpl_func, "Subtracting bias from %s", filename);
-        
-        fors_image_subtract(image, bias);
-        
-        assure( !cpl_error_get_code(), return image, 
-                "Bias subtraction failed" );
-
-        if (has_overscans) {
-
-            /*
-             * Overscan correction
-             */
-
-            ocorr -= cpl_image_get_median(bias->data);
-
-            cpl_msg_info(cpl_func, 
-                         "Overscan correction applied: %.2f ADUs", ocorr);
-
-            fors_image_subtract_scalar(image, ocorr, -1);
-        }
-        
-        /* Variance is now (ron**2  +  bias_noise**2).
-           Add photonic noise:
-           
-           variance := variance + |f|/conad
-        */
-
-        double conad = 1.0 / frame_setting->average_gain;
-
-        temp = cpl_image_divide_scalar_create(image->data, conad);
-        cpl_image_abs(temp);
-
-        cpl_image_add(image->variance, temp);
-    }
     
     cleanup;
     return image;
@@ -573,6 +360,7 @@ fors_image_load(const cpl_frame *frame, const fors_image *bias,
 #define cleanup \
 do { \
     cpl_image_delete(sigma); \
+    cpl_propertylist_delete(extension_header); \
 } while(0)
 /**
  * @brief    Save image
@@ -601,6 +389,9 @@ fors_image_save(const fors_image *image, const cpl_propertylist *header,
     
     sigma = cpl_image_power_create(image->variance, 0.5);
     /* This would probably be faster if sqrt() is used rather than pow */
+    extension_header = cpl_propertylist_new();
+    cpl_propertylist_append_string(extension_header, 
+                                   "EXTNAME", "IMAGE.ERR");
 
     cpl_image_save(sigma, filename, CPL_BPP_IEEE_FLOAT, extension_header,
                    CPL_IO_EXTEND);
@@ -703,7 +494,7 @@ fors_image_save_sex(const fors_image *image, const cpl_propertylist *header,
  * @param   image      image
  * @return  width
  **/
-int fors_image_get_size_x(const fors_image *image)
+cpl_size fors_image_get_size_x(const fors_image *image)
 {
     assure( image != NULL, return -1, NULL );
     return cpl_image_get_size_x(image->data);
@@ -716,7 +507,7 @@ int fors_image_get_size_x(const fors_image *image)
  * @param   image      image
  * @return  width
  **/
-int fors_image_get_size_y(const fors_image *image)
+cpl_size fors_image_get_size_y(const fors_image *image)
 {
     assure( image != NULL, return -1, NULL );
     return cpl_image_get_size_y(image->data);
@@ -1220,7 +1011,7 @@ void fors_image_crop(fors_image *image,
     assure( 1 <= xlo && xlo <= xhi && xhi <= fors_image_get_size_x(image) &&
             1 <= ylo && ylo <= yhi && yhi <= fors_image_get_size_y(image),
             return, "Cannot extraction region (%d, %d) - (%d, %d) of "
-            "%dx%d image",
+            "%"CPL_SIZE_FORMAT"x%"CPL_SIZE_FORMAT" image",
             xlo, ylo, xhi, yhi,
             fors_image_get_size_x(image),
             fors_image_get_size_y(image));
@@ -1967,6 +1758,37 @@ void fors_image_draw(fors_image *image, int type,
     return;
 }
 
+hdrl_imagelist * fors_image_list_to_hdrl(const fors_image_list * imalist)
+{
+    int i;
+    hdrl_imagelist * images_hdrl = hdrl_imagelist_new();
+    const fors_image * target = fors_image_list_first_const(imalist);
+    for(i = 0 ; i < fors_image_list_size(imalist); ++i)
+    {
+        const cpl_image * ima_data  = target->data;
+        cpl_image * ima_error = cpl_image_power_create(target->variance, 0.5);
+        cpl_mask * old_bpm = cpl_image_set_bpm(ima_error, 
+                               cpl_mask_duplicate(cpl_image_get_bpm(ima_data)));
+        cpl_mask_delete(old_bpm);
+        hdrl_image * ima_hdrl = hdrl_image_create(ima_data, ima_error);
+        hdrl_imagelist_set(images_hdrl, ima_hdrl, 
+                           hdrl_imagelist_get_size(images_hdrl));
+        target = fors_image_list_next_const(imalist);
+    }
+    
+    return images_hdrl;
+}
+
+fors_image * fors_image_from_hdrl(const hdrl_image * image)
+{
+    const cpl_image * data = hdrl_image_get_image_const(image);
+    cpl_image * variance = cpl_image_power_create
+            (hdrl_image_get_image_const(image), 2);
+    fors_image * ima = fors_image_new(cpl_image_duplicate(data), variance);
+    return ima;
+}
+
+
 #define LIST_DEFINE
 #undef LIST_ELEM
 #define LIST_ELEM fors_image
