@@ -41,6 +41,18 @@ template<typename T>
 void vector_smooth(std::vector<T>& input, 
                    std::vector<T>& input_err,
                    size_t smooth_size);
+
+template<typename T>
+void vector_smooth(std::vector<T>& input, 
+                   size_t smooth_size);
+
+template<typename T>
+void vector_cubicspline_fit(std::vector<T>& input, 
+                            size_t nknots, double threshold);
+
+template<typename T>
+void vector_pol_fit(std::vector<T>& input, int polyorder, double threshold);
+
 }
 
 
