@@ -71,7 +71,7 @@ static char fors_align_sky_description[] =
 
 #define fors_align_sky_exit(message)          \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(wavemap);                    \
 cpl_image_delete(coordinate);                 \
 cpl_image_delete(rainbow);                    \
@@ -91,7 +91,7 @@ return -1;                                    \
 
 #define fors_align_sky_exit_memcheck(message)   \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free wavemap (%p)\n", wavemap);         \
 cpl_image_delete(wavemap);                      \
 printf("free coordinate (%p)\n", coordinate);   \
