@@ -77,7 +77,7 @@ static char fors_detect_spectra_description[] =
 
 #define fors_detect_spectra_exit(message)     \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(spectra);                    \
 cpl_image_delete(checkwave);                  \
 cpl_image_delete(refimage);                   \
@@ -94,7 +94,7 @@ return -1;                                    \
 
 #define fors_detect_spectra_exit_memcheck(message) \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free spectra (%p)\n", spectra);         \
 cpl_image_delete(spectra);                      \
 printf("free checkwave (%p)\n", checkwave);     \
