@@ -100,7 +100,7 @@ static char fors_extract_description[] =
 
 #define fors_extract_exit(message)            \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_free(exptime);                            \
 cpl_free(instrume);                           \
 cpl_image_delete(dummy);                      \
@@ -137,7 +137,7 @@ return -1;                                    \
 
 #define fors_extract_exit_memcheck(message)            \
 {                                                      \
-if (message) cpl_msg_info(recipe, message);            \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);            \
 printf("free exptime (%p)\n", exptime);                \
 cpl_free(exptime);                                     \
 printf("free instrume (%p)\n", instrume);              \
