@@ -103,7 +103,7 @@ static char fors_pmos_calib_description[] =
 
 #define fors_pmos_calib_exit(message)              \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_free(instrume);                           \
 cpl_free(fiterror);                           \
 cpl_free(fitlines);                           \
@@ -152,7 +152,7 @@ return -1;                                    \
 
 #define fors_pmos_calib_exit_memcheck(message)       \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free instrume (%p)\n", instrume);       \
 cpl_free(instrume);                             \
 printf("free pipefile (%p)\n", pipefile);       \
