@@ -72,7 +72,7 @@ static char fors_resample_description[] =
 
 #define fors_resample_exit(message)           \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(spectra);                    \
 cpl_image_delete(mapped);                     \
 cpl_table_delete(grism_table);                \
@@ -85,7 +85,7 @@ return -1;                                    \
 
 #define fors_resample_exit_memcheck(message)    \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free spectra (%p)\n", spectra);         \
 cpl_image_delete(spectra);                      \
 printf("free mapped (%p)\n", mapped);           \
