@@ -72,7 +72,7 @@ static char fors_subtract_sky_description[] =
 
 #define fors_subtract_sky_exit(message)       \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(spectra);                    \
 cpl_image_delete(skymap);                     \
 cpl_table_delete(grism_table);                \
@@ -86,7 +86,7 @@ return -1;                                    \
 
 #define fors_subtract_sky_exit_memcheck(message)   \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free spectra (%p)\n", spectra);         \
 cpl_image_delete(spectra);                      \
 printf("free skymap (%p)\n", skymap);           \
