@@ -63,7 +63,7 @@ static char fors_sumflux_description[] =
 
 #define fors_sumflux_exit(message)            \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_free(instrume);                           \
 cpl_free(pipefile);                           \
 cpl_image_delete(master_bias);                \
@@ -77,7 +77,7 @@ return -1;                                    \
 
 #define fors_sumflux_exit_memcheck(message)   \
 {                                             \
-if (message) cpl_msg_info(recipe, message);   \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);   \
 cpl_free(instrume);                           \
 cpl_free(pipefile);                           \
 cpl_image_delete(master_bias);                \
