@@ -82,7 +82,7 @@ static char fors_trace_flat_description[] =
 
 #define fors_trace_flat_exit(message)         \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(master_flat);                \
 cpl_image_delete(spatial);                    \
 cpl_image_delete(coordinate);                 \
@@ -98,7 +98,7 @@ return -1;                                    \
 
 #define fors_trace_flat_exit_memcheck(message)  \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free master_flat (%p)\n", master_flat); \
 cpl_image_delete(master_flat);                  \
 printf("free spatial (%p)\n", spatial);         \
