@@ -71,7 +71,7 @@ static char fors_wave_calib_description[] =
 
 #define fors_wave_calib_exit(message)         \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(spectra);                    \
 cpl_image_delete(spatial);                    \
 cpl_image_delete(rainbow);                    \
@@ -94,7 +94,7 @@ return -1;                                    \
 
 #define fors_wave_calib_exit_memcheck(message)  \
 {                                               \
-if (message) cpl_msg_info(recipe, message);     \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);     \
 printf("free spectra (%p)\n", spectra);         \
 cpl_image_delete(spectra);                      \
 printf("free spatial (%p)\n", spatial);         \
