@@ -67,7 +67,7 @@ static char fors_wave_calib_lss_description[] =
 
 #define fors_wave_calib_lss_exit(message)     \
 {                                             \
-if (message) cpl_msg_error(recipe, message);  \
+if ((const char *)message != NULL) cpl_msg_error(recipe, message);  \
 cpl_image_delete(spectra);                    \
 cpl_image_delete(residual);                   \
 cpl_image_delete(rectified);                  \
@@ -88,7 +88,7 @@ return -1;                                    \
 
 #define fors_wave_calib_lss_exit_memcheck(message)  \
 {                                                   \
-if (message) cpl_msg_info(recipe, message);         \
+if ((const char *)message != NULL) cpl_msg_info(recipe, message);         \
 printf("free spectra (%p)\n", spectra);             \
 cpl_image_delete(spectra);                          \
 printf("free residual (%p)\n", residual);           \
