@@ -1,6 +1,7 @@
 # import the needed modules
 try:
   import reflex
+  import matplotlib.gridspec
   import_sucess = True
 
 #NOTE for developers: 
@@ -35,7 +36,11 @@ try:
     # sof contains all the set of frames
     def readFitsData(self, fitsFiles):
       #Initialise the objects to read/display
-      self.sci_reduced    = None
+      self.sci_mapped    = None
+      self.sci_extracted = None
+      self.skylines_off  = None
+      self.object_table  = None
+      self.objid = 1
 
       #Read all the products
       frames = dict()
@@ -45,10 +50,23 @@ try:
         category = frame.category
         frames[category] = frame
 
+      for inst_mode in ('MXU', 'MOS', 'LSS', 'LONG_MOS', 'LONG_MXU') :
+        if 'OBJECT_TABLE_SCI_'+inst_mode in frames :
+          self.object_table = frames['OBJECT_TABLE_SCI_'+inst_mode]
 
       for inst_mode in {'MXU', 'MOS', 'LSS', 'LONG_MOS', 'LONG_MXU'} :
-        if frames.has_key('MAPPED_SCI_'+inst_mode):
-          self.sci_reduced   = PlotableMappedScience(frames["MAPPED_SCI_"+inst_mode])
+        if 'MAPPED_SCI_'+inst_mode in frames:
+          self.sci_mapped   = PlotableMappedScience(frames["MAPPED_SCI_"+inst_mode],
+                                                    self.object_table)
+          
+      for inst_mode in {'MXU', 'MOS', 'LSS', 'LONG_MOS', 'LONG_MXU'} :
+        if 'REDUCED_SCI_'+inst_mode in frames:
+          self.sci_extracted   = PlotableExtractedScience(frames["REDUCED_SCI_"+inst_mode])
+          
+      for inst_mode in {'MXU', 'MOS', 'LSS', 'LONG_MOS', 'LONG_MXU'} :
+        if 'SKY_SHIFTS_SLIT_SCI_'+inst_mode in frames:
+          self.skylines_off   = PlotableSkylinesOffsets(frames["SKY_SHIFTS_SLIT_SCI_"+inst_mode])
+          
 
                   
     # This function creates all the subplots. It is responsible for the plotting 
@@ -57,8 +75,15 @@ try:
     # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
     # Note also that the last one is actually a box with text, no graphs.
     def addSubplots(self, figure):
-      if self.sci_reduced is not None :
-        self.subplot_sci_reduced    = figure.add_subplot(1,1,1)
+      if self.sci_mapped is not None :
+        if self.skylines_off is not None :
+          gs = matplotlib.gridspec.GridSpec(7,7)
+          self.subplot_sci_mapped    = figure.add_subplot(gs[0:2,:])
+          self.subplot_sci_extracted = figure.add_subplot(gs[3:5,:])
+          self.subplot_skylines_off  = figure.add_subplot(gs[6,:])
+        else :
+          self.subplot_sci_mapped    = figure.add_subplot(2,1,1)
+          self.subplot_sci_extracted = figure.add_subplot(2,1,2)
       else : 
         self.subtext_nodata      = figure.add_subplot(1,1,1)
           
@@ -69,25 +94,91 @@ try:
     # One might be tempted to merge addSubplots() and plotProductsGraphics().
     # There is a reason not to do it: addSubplots() is called only once at
     # startup, while plotProductsGraphics() is called always there is a resize.
-    def plotProductsGraphics(self, figure, canvas):
-      if self.sci_reduced is not None :
+    def plotProductsGraphics(self):
+      if self.sci_mapped is not None :
 
         #Reduced science
-        title_sci_reduced   = 'Reduced mapped science' 
-        tooltip_sci_reduced ="""Reduced mapped science."""
-        self.sci_reduced.plot(self.subplot_sci_reduced, title_sci_reduced, 
-                               tooltip_sci_reduced)
+        self.plotSciMapped()
+
+        #Extracted science
+        if self.sci_extracted is not None :
+            self.plotSciExtracted()
+        else:
+            self.showNoExtracted()
+
+        #Sky lines offset
+        self.plotSkyLinesOffsets()
 
-        
       else :
         #Data not found info
-        self.subtext_nodata.set_axis_off()
-        self.text_nodata = 'Science data not found in the products (PRO.CATG=)' 
-        self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
-                                 ha='left', va='center', alpha=1.0)
-        self.subtext_nodata.tooltip='Science data not found in the products'
- 
+        self.showNoData()
   
+    def plotSciMapped(self) :
+        title_sci_mapped   = 'Mapped science (not flux-calibrated)'
+        tooltip_sci_mapped = """Mapped science (not flux-calibrated).
+The regions of extracted spectra are marked by red 
+(upper boundary of extraction region) 
+and blue (lower boundary) lines. 
+By middle-clicking within one of these regions,
+you select an extracted spectrum,
+which will be plotted below."""
+        self.sci_mapped.plot(self.subplot_sci_mapped, title_sci_mapped,
+                             tooltip_sci_mapped)
+
+    def plotSciExtracted(self) :
+        title_sci_extracted = 'Extracted science spectrum (not flux-calibrated)'
+        tooltip_sci_extracted ="""Extracted science spectrum (not flux-calibrated).
+This is the sky-subtracted spectrum as extracted by the pipeline."""
+        self.subplot_sci_extracted.clear()
+        self.sci_extracted.plot(self.subplot_sci_extracted, title_sci_extracted,
+                                tooltip_sci_extracted)
+
+    def showNoExtracted(self) :
+      #Data not found info
+      self.subplot_sci_extracted.set_axis_off()
+      self.text_nodata = 'No objects found. There are no extracted spectra.'
+      self.subplot_sci_extracted.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
+                               ha='left', va='center', alpha=1.0)
+      self.subplot_sci_extracted.tooltip='No objects found. There are no extracted spectra'
+
+
+    def plotSkyLinesOffsets(self) :
+      if self.skylines_off is not None :
+        title_skylines_off   = 'Sky lines offsets'
+        tooltip_skylines_off ="""Sky lines offsets.
+This plots shows the offsets of certain sky lines
+(defined in the MASTER_SKYLINECAT table or 
+pipeline values as given in the User Manual) 
+relative to their reference position. 
+The offsets are adjusted by default by 
+a constant shift (--skyalign parameter)."""
+        self.skylines_off.plot(self.subplot_skylines_off, title_skylines_off,
+                               tooltip_skylines_off)
+
+    def showNoData(self) :
+      #Data not found info
+      self.subtext_nodata.set_axis_off()
+      self.text_nodata = 'Science data not found in the products (PRO.CATG=)'
+      self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
+                               ha='left', va='center', alpha=1.0)
+      self.subtext_nodata.tooltip='Science data not found in the products'
+
+    def plotWidgets(self) :
+      widgets = list()
+      if self.sci_mapped is not None :
+        # Clickable subplot
+        self.clickablesmapped = InteractiveClickableSubplot(
+          self.subplot_sci_mapped, self.setExtractedObject)
+        widgets.append(self.clickablesmapped)
+      return widgets
+    
+    def setExtractedObject(self, point) :
+      obj = self.sci_mapped.getObjectInPosition(point.ydata)
+      if obj != -1 :
+        self.sci_extracted.selectObject(obj)
+      self.plotSciExtracted()
+        
+                
     # This function specifies which are the parameters that should be presented
     # in the window to be edited.
     # Note that the parameter has to be also in the in_sop port (otherwise it 
